@@ -2,11 +2,13 @@
 from __future__ import print_function
 import os
 # import IgorModel
-
+flag_verbose = False
 import requests
 from bs4 import BeautifulSoup
 from Bio import SeqIO
 
+from .utils import *
+
 try:
     from StringIO import StringIO # Python 2
 except ImportError:
@@ -85,20 +87,15 @@ def load_records_from_fasta(filename:str):
     # flnGenomicJs = DIR_REF_GENOME + "genomicJs.fasta"
     return list(SeqIO.parse(filename, "fasta"))
 
-def makeDirectories(gene: str, specie: str, modelspath=None):
-    if modelspath is None:
-        modelspath = "models"
-
-    os.system("mkdir -p " + modelspath)
-    os.system("mkdir -p " + modelspath + "/" + specie)
-    os.system("mkdir -p " + modelspath + "/" + specie + "/" + gene)
-    os.system("mkdir -p " + modelspath + "/" + specie + "/" + gene + "/ref_genome")
-    os.system("mkdir -p " + modelspath + "/" + specie + "/" + gene + "/ref_genome")
-    os.system("mkdir -p " + modelspath + "/" + specie + "/" + gene + "/models")
 
 def get_gene_template(specie: str, gene: str, modelspath=None, filename=None, imgt_genedb=imgt_params['url.genedb']):
     """
     Download genomic template according to specie and gene from imgt database
+
+    :param species: Species name in IMGT nomenclature.
+    :type species: str
+    :param gene: IMGT gene nomenclature like TRAV, IGHJ, etc.
+    :type species: str
     :return : string of the requested link
     """
     try:
@@ -112,33 +109,50 @@ def get_gene_template(specie: str, gene: str, modelspath=None, filename=None, im
     except Exception as e:
         raise e
 
-def download_gene_template(specie: str, gene: str, modelspath=None, filename=None, imgt_genedb=imgt_params['url.genedb']):
+def download_gene_template(specie: str, gene: str, ref_genes_path=None, modelspath=None, filename=None, imgt_genedb=imgt_params['url.genedb']):
     """
     Create a file in IGoR default format
     and returns output filename.
     :param specie: IMGT specie nomenclature use "+" instead of a space " " Mus+musculus
+    :type species: str
     :param gene: IMGT gene nomenclature like TRAV, IGHJ, etc
+    :type species: str
+    :param modelspath: root paths for all models.
+    :type modelspath: str, optional
+    :param filename: gene template fasta filename.
+    :type filename: str, optional
+    :param imgt_genedb: Url of IMGT GeneDB web application.
+    :type imgt_genedb: str
     """
     # 3. Write it in a fasta file.
     records = get_gene_template(specie, gene, imgt_genedb=imgt_genedb)
 
     if gene[-1] in ['V', 'D', 'J']:
-        ref_genes_path = ""
-        if modelspath is None:
-            modelspath = "models"
+        if ref_genes_path is None:
+            ref_genes_path = ""
+
+            if modelspath is None:
+                modelspath = "models"
+
             gene_dir = gene[:-1]
-            makeDirectories(gene_dir, specie, modelspath=modelspath)
+            make_igor_directories(gene_dir, specie, modelspath=modelspath)
             ref_genes_path = modelspath + "/" + specie + "/" + gene_dir + "/ref_genome/"
+        else:
+            # create directory
+            ref_genes_path = ref_genes_path + "/"
+            os.system("mkdir -p " + ref_genes_path)
 
         if filename is None :
             filename = ref_genes_path + "genomic" + gene[-1] + "s__imgt.fasta"
             #filename = modelspath + "genomic__" + specie + "_" + gene + ".fasta"
+
     #flnGenomicJs = DIR_REF_GENOME + "genomicJs.fasta"
     flnGenome = save_records2fasta(records, filename)
     fln_dict = dict()
     fln_dict["fln"+gene[-1]+"Genome"] = flnGenome
     fln_dict['ref_genes_path'] = ref_genes_path
     fln_dict['modelspath'] = modelspath
+    fln_dict['records'] = records
     return fln_dict
 
 def genKey(seqDescription):
@@ -193,29 +207,38 @@ def get_gene_anchors(specie: str, gene: str, imgtlabel: str, modelspath=None, fi
     except Exception as e:
         raise e
 
-def download_genes_anchors(specie: str, chain: str, flnVGenome, flnJGenome, modelspath=None, imgt_genedb=imgt_params['url.genedb']):
-    # For this method anchors will be in the same directory modelspath+"/"+specie+"/"+chain+
-    V_dict = download_Vgene_anchors(specie, chain, flnVGenome, modelspath=modelspath, imgt_genedb=imgt_genedb)
-    J_dict = download_Jgene_anchors(specie, chain, flnJGenome, modelspath=modelspath, imgt_genedb=imgt_genedb)
+def download_genes_anchors(specie: str, chain: str, flnVGenome, flnJGenome, ref_genes_path=None, modelspath=None, imgt_genedb=imgt_params['url.genedb']):
+    """
+    Download gene anchors from IMGT website in a IGoR structure directory (modelspath+"/"+specie+"/"+chain)
+    """
+
+    V_dict = download_Vgene_anchors(specie, chain, flnVGenome, ref_genes_path=ref_genes_path, modelspath=modelspath, imgt_genedb=imgt_genedb)
+    J_dict = download_Jgene_anchors(specie, chain, flnJGenome, ref_genes_path=ref_genes_path, modelspath=modelspath, imgt_genedb=imgt_genedb)
     Anchors_dict = {**V_dict, **J_dict}
     return Anchors_dict
 
-def download_Vgene_anchors(specie: str, chain: str, flnVGenome, modelspath=None, imgt_genedb=imgt_params['url.genedb']):
+def download_Vgene_anchors(specie: str, chain: str, flnVGenome, ref_genes_path=None, modelspath=None, imgt_genedb=imgt_params['url.genedb'], sep=';'):
     #records = get_gene_template(specie, gene, imgt_genedb=imgt_genedb)
-    if modelspath is None:
-        modelspath = "models"
-    makeDirectories(chain, specie, modelspath=modelspath)
-    ref_genes_path = modelspath + "/" + specie + "/" + chain + "/ref_genome/"
+    if ref_genes_path is None:
+        ref_genes_path = ""
+        if modelspath is None:
+            modelspath = "models"
+        make_igor_directories(chain, specie, modelspath=modelspath)
+        ref_genes_path = modelspath + "/" + specie + "/" + chain + "/ref_genome/"
+    else:
+        ref_genes_path = ref_genes_path + "/"
+
 
     # generate dictionaries with the anchors
     # 2nd-CYS
     urlV_2CYS = get_genedb_query81_imgtlabel(specie, chain + "V", imgtlabel="2nd-CYS", imgt_genedb=imgt_genedb)
     dictV_2CYS = genAnchDict(urlV_2CYS)
 
-    flnAnchors = ref_genes_path +  "V_gene_CDR3_anchors__imgt.csv"
+    flnAnchors = ref_genes_path + "V_gene_CDR3_anchors__imgt.csv"
+    CSVDELIM = sep
     ofileAnch = open(flnAnchors, "w")
-    ofileAnch.write("gene;anchor_index;function" + "\n")
-    CSVDELIM = ";"
+    ofileAnch.write("gene"+CSVDELIM+"anchor_index"+CSVDELIM+"gfunction" + "\n")
+
 
     Vrecords = SeqIO.parse(flnVGenome, "fasta")
     for rec in Vrecords:
@@ -228,7 +251,8 @@ def download_Vgene_anchors(specie: str, chain: str, flnVGenome, modelspath=None,
             seqFunc = getFunction(rec.description)
             ofileAnch.write(rec.description + CSVDELIM + str(posAnch) +  CSVDELIM+str(seqFunc)+"\n")
         else:
-            print("No anchor is found for : " + rec.description)
+            if flag_verbose:
+                print("No anchor is found for : " + rec.description)
 
     ofileAnch.close()
     fln_dict = dict()
@@ -238,14 +262,58 @@ def download_Vgene_anchors(specie: str, chain: str, flnVGenome, modelspath=None,
     fln_dict['modelspath'] = modelspath
     return fln_dict
 
-def download_Jgene_anchors(specie: str, chain: str, flnJGenome, modelspath=None, imgt_genedb=imgt_params['url.genedb']):
-    # records = get_gene_template(specie, gene, imgt_genedb=imgt_genedb)
+def download_Vgene_anchors_bk(specie: str, chain: str, modelspath=None, imgt_genedb=imgt_params['url.genedb']):
+    #records = get_gene_template(specie, gene, imgt_genedb=imgt_genedb)
     if modelspath is None:
         modelspath = "models"
-
-    makeDirectories(chain, specie, modelspath=modelspath)
+    make_igor_directories(chain, specie, modelspath=modelspath)
     ref_genes_path = modelspath + "/" + specie + "/" + chain + "/ref_genome/"
 
+    # generate dictionaries with the anchors
+    # 2nd-CYS
+    urlV_2CYS = get_genedb_query81_imgtlabel(specie, chain + "V", imgtlabel="2nd-CYS", imgt_genedb=imgt_genedb)
+    dictV_2CYS = genAnchDict(urlV_2CYS)
+
+    flnAnchors = ref_genes_path + "V_gene_CDR3_anchors__imgt.csv"
+    return dictV_2CYS
+    # ofileAnch = open(flnAnchors, "w")
+    # CSVDELIM = ";"
+    # ofileAnch.write("gene"+CSVDELIM+"anchor_index"+CSVDELIM+"function" + "\n")
+
+    # Vrecords = SeqIO.parse(flnVGenome, "fasta")
+    # for rec in Vrecords:
+    #     key = genKey(rec.description)
+    #     # Initiate to avoid bad designation.
+    #     posV_2CYS = -1
+    #     if key in dictV_2CYS.keys():
+    #         posV_2CYS = dictV_2CYS[key] - getStartPos(rec.description)
+    #         posAnch = posV_2CYS
+    #         seqFunc = getFunction(rec.description)
+    #         ofileAnch.write(rec.description + CSVDELIM + str(posAnch) + CSVDELIM+str(seqFunc)+"\n")
+    #     else:
+    #         print("No anchor is found for : " + rec.description)
+    #
+    # ofileAnch.close()
+    # fln_dict = dict()
+    # fln_dict['flnVGenome'] = flnVGenome
+    # fln_dict['flnVAnchors'] = flnAnchors
+    # fln_dict['ref_genes_path'] = ref_genes_path
+    # fln_dict['modelspath'] = modelspath
+    # return fln_dict
+
+
+def download_Jgene_anchors(specie: str, chain: str, flnJGenome, ref_genes_path=None, modelspath=None, imgt_genedb=imgt_params['url.genedb']):
+    # records = get_gene_template(specie, gene, imgt_genedb=imgt_genedb)
+    if ref_genes_path is None:
+        ref_genes_path = ""
+        if modelspath is None:
+            modelspath = "models"
+            make_igor_directories(chain, specie, modelspath=modelspath)
+        ref_genes_path = modelspath + "/" + specie + "/" + chain + "/ref_genome/"
+    else:
+        ref_genes_path = ref_genes_path + "/"
+
+
     # J-PHE
     urlJ_PHE = get_genedb_query81_imgtlabel(specie, chain + "J", imgtlabel="J-PHE", imgt_genedb=imgt_genedb)
     dictJ_PHE = genAnchDict(urlJ_PHE)
@@ -281,7 +349,8 @@ def download_Jgene_anchors(specie: str, chain: str, flnJGenome, modelspath=None,
             ofileAnch.write(rec.description + CSVDELIM + str(posAnch) + CSVDELIM + str(seqFunc) + "\n")
             # ofileAnch.write(rec.description + CSVDELIM + str(posAnch) + "\n")
         else:
-            print("No anchor is found for : " + rec.description)
+            if flag_verbose:
+                print("No anchor is found for : " + rec.description)
 
     ofileAnch.close()
     fln_dict = dict()
@@ -292,7 +361,15 @@ def download_Jgene_anchors(specie: str, chain: str, flnJGenome, modelspath=None,
     return fln_dict
 
 ############ Donwload all genetic information if VDJ or VJ
-def download_ref_genome_VDJ(species: str, chain: str, **kwargs):
+def download_ref_genome_VDJ(species: str, chain: str, dropna=False, **kwargs):
+    """
+    Return a dictionary with genomics dataframes and also save files in a IGoR directory structure.
+    :param species: IMGT species name
+    :param chain: IMGT chain receptor name
+    :param dropna: Remove rows with no defined values in any column
+    :return : dictionary of pandas DataFrame with
+    """
+    # flag_verbose = False
     dictVGenome = download_gene_template(species, chain + 'V', **kwargs)
     dictDGenome = download_gene_template(species, chain + 'D', **kwargs)
     dictJGenome = download_gene_template(species, chain + 'J', **kwargs)
@@ -320,7 +397,8 @@ def download_ref_genome_VDJ(species: str, chain: str, **kwargs):
         # if v_genome in list2CYS:
             v_genomes_trim_list.append(v_genome)
         else:
-            print(v_genome.description)
+            if flag_verbose:
+                print(v_genome.description)
     save_records2fasta(v_genomes_trim_list, flnVGenome + "_trim")
 
     # J-PHE
@@ -343,12 +421,14 @@ def download_ref_genome_VDJ(species: str, chain: str, **kwargs):
         # elif j_genome in listJ_TRP:
             j_genomes_trim_list.append(j_genome)
         else:
-            print(j_genome.description)
+            if flag_verbose:
+                print(j_genome.description)
 
     save_records2fasta(j_genomes_trim_list, flnJGenome + "_trim")
-    print("----------------------")
-    print("Genomic VDJ templates in files: ")
-    print(flnVGenome, flnDGenome, flnJGenome)
+    if flag_verbose:
+        print("----------------------")
+        print("Genomic VDJ templates in files: ")
+        print(flnVGenome, flnDGenome, flnJGenome)
     dict_V = gen_short_names(flnVGenome, flnAnchors=Anchors_dict["flnVAnchors"])
     dict_J = gen_short_names(flnJGenome, flnAnchors=Anchors_dict["flnJAnchors"])
     dict_D = gen_short_names(flnDGenome)
@@ -363,9 +443,42 @@ def download_ref_genome_VDJ(species: str, chain: str, **kwargs):
     shutil.copy(dict_D["genomics"], D_path + "/genomicDs.fasta")
     shutil.copy(dict_V["anchors"], V_path + "/V_gene_CDR3_anchors.csv")
     shutil.copy(dict_J["anchors"], J_path + "/J_gene_CDR3_anchors.csv")
+
+    from .utils import get_dataframe_from_fasta_and_csv_anchors
+    df_genomics_dict = dict()
+    df_genomics_dict['V'] = get_dataframe_from_fasta_and_csv_anchors(dict_V["genomics"], dict_V["anchors"])
+    df_genomics_dict['D'] = get_dataframe_from_fasta_and_csv_anchors(dict_D["genomics"])
+    df_genomics_dict['J'] = get_dataframe_from_fasta_and_csv_anchors(dict_J["genomics"], dict_J["anchors"])
+
+    if dropna:
+        df_genomics_dict['V'].dropna(inplace=True)
+        df_genomics_dict['V']['anchor_index'] = df_genomics_dict['V']['anchor_index'].astype(int)
+        df_genomics_dict['V'].set_index(np.arange(len(df_genomics_dict['V'])), inplace=True)
+
+        df_genomics_dict['J'].dropna(inplace=True)
+        df_genomics_dict['J']['anchor_index'] = df_genomics_dict['J']['anchor_index'].astype(int)
+        df_genomics_dict['J'].set_index(np.arange(len(df_genomics_dict['J'])), inplace=True)
+
+    return df_genomics_dict
+
     # print(kwargs)
 
-def download_ref_genome_VJ(species: str, chain: str, **kwargs):
+def download_ref_genome_VJ(species: str, chain: str, dropna=False, **kwargs):
+    """Download gene templates and anchors from IMGT and creates
+    different files with original and short names.
+
+    :param species: Species name in IMGT nomenclature,
+    :type species: str
+    :param chain: Chain name in IMGT nomenclature,
+    :type chain: str
+    :param modelspath: root paths for all models.
+    :type modelspath: str, optional
+    :param filename: gene template fasta filename.
+    :type filename: str, optional
+    :param imgt_genedb: Url of IMGT GeneDB web application.
+    :type imgt_genedb: str
+    :param dropna: Remove rows with no defined values in any column
+    """
     dictVGenome = download_gene_template(species, chain + 'V', **kwargs)
     dictJGenome = download_gene_template(species, chain + 'J', **kwargs)
 
@@ -433,6 +546,94 @@ def download_ref_genome_VJ(species: str, chain: str, **kwargs):
     shutil.copy(dict_V["anchors"], V_path + "/V_gene_CDR3_anchors.csv")
     shutil.copy(dict_J["anchors"], J_path + "/J_gene_CDR3_anchors.csv")
 
+    from .utils import get_dataframe_from_fasta_and_csv_anchors
+    df_genomics_dict = dict()
+    df_genomics_dict['V'] = get_dataframe_from_fasta_and_csv_anchors(dict_V["genomics"], dict_V["anchors"])
+    df_genomics_dict['J'] = get_dataframe_from_fasta_and_csv_anchors(dict_J["genomics"], dict_J["anchors"])
+
+    if dropna:
+        df_genomics_dict['V'].dropna(inplace=True)
+        df_genomics_dict['V']['anchor_index'] = df_genomics_dict['V']['anchor_index'].astype(int)
+        df_genomics_dict['V'].set_index(np.arange(len(df_genomics_dict['V'])), inplace=True)
+
+        df_genomics_dict['J'].dropna(inplace=True)
+        df_genomics_dict['J']['anchor_index'] = df_genomics_dict['J']['anchor_index'].astype(int)
+        df_genomics_dict['J'].set_index(np.arange(len(df_genomics_dict['J'])), inplace=True)
+
+    return df_genomics_dict
+
+
+def download_ref_genome(species: str, chain: str, dropna=False, **kwargs):
+    """
+    Returns dictionary with genomic templates V(D)J
+    :param species: IMGT species check get_species_list() to get list of possible names
+    :param chain: IMGT chain, if ['TRB', 'IGH'] returns VDJ dict and VJ for ['TRA', 'IGL'].
+    :param dropna: Drop NaN in dataframes with new ids
+    :return: genomic templates dictionary
+    """
+    # FIXME: IN DEV
+    #     TODO: CHECK THE IMGT STANDARDS TO DEFINE IG AND TR
+    ref_genome_dict = dict()
+    if chain in ['TRB', 'IGH']:
+        ref_genome_dict = download_ref_genome_VDJ(species, chain, dropna=dropna, **kwargs)
+    elif chain in ['TRA', 'IGL']:
+        ref_genome_dict = download_ref_genome_VJ(species, chain, dropna=dropna, **kwargs)
+    else:
+        ref_genome_dict = None
+
+    return ref_genome_dict
+
+
+def get_dict_from_imgt_description(str_description):
+    """
+    Return an OderedDict with following the fields
+    The IMGT FASTA header of nucleotide IMGT  reference sequences contains 15 fields separated by '|':
+    1. IMGT/LIGM-DB accession number(s)
+    2. IMGT gene and allele name
+    3. species
+    4. IMGT allele functionality
+    5. exon(s), region name(s), or extracted label(s)
+    6. start and end positions in the IMGT/LIGM-DB accession number(s)
+    7. number of nucleotides in the IMGT/LIGM-DB accession number(s)
+    8. codon start, or 'NR' (not relevant) for non coding labels
+    9. +n: number of nucleotides (nt) added in 5' compared to the corresponding label extracted from IMGT/LIGM-DB
+    10. +n or -n: number of nucleotides (nt) added or removed in 3' compared to the corresponding label extracted from IMGT/LIGM-DB
+    11. +n, -n, and/or nS: number of added, deleted, and/or substituted nucleotides to correct sequencing errors, or 'not corrected' if non corrected sequencing errors
+    12. number of amino acids (AA): this field indicates that the sequence is in amino acids
+    13. number of characters in the sequence: nt (or AA)+IMGT gaps=total
+    14. partial (if it is)
+    15. reverse complementary (if it is)
+    """
+
+    description_field_list = ["accession_number",  # 1
+             "allele_name",  # 2
+             "species",  # 3
+             "function",  # 4
+             "exon_region_name_label",  # 5
+             "start_end",  # 6
+             "n_nt",  # 7
+             "codon_start",  # 8
+             "n_nt_added_5p", # 9
+             "n_nt_added_removed_3p", # 10
+             "n_nt_added_deleted_substituted",  # 11
+             "n_aa",  # 12
+             "n_char_plus_gaps",  # 13
+             "partial",  # 14
+             "reverse_complementary"  # 15
+             ]
+    from collections import OrderedDict
+    dicto = OrderedDict()
+    description_list = str_description.split("|")
+    print(len(description_field_list), description_field_list)
+    print(len(description_list), description_list)
+    dicto["description"] = str_description
+    for ii, elem in enumerate(description_field_list):
+        print(ii, elem, description_list[ii])
+        dicto[elem] = description_list[ii]
+
+    return dicto
+
+
 
 def make_VDJ_model():
     from .IgorIO import IgorModel_Parms
